/*===========================================================================
 * DESCRIPTION:
 *
 * This file implements CAN functions that mimick the STMicroelectronics'
 * STM8S library, but specifically for the CAN peripheral.
 * As CAN support is not yet available from the STM8S library, Raisonance
 * makes this file available for your convenience, although NO SUPPORT will
 * be provided for it.
 *
 *---------------------------------------------------------------------------
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *---------------------------------------------------------------------------
 * Copyright (c) Raisonance S.A.S., 2005-2008
 *==========================================================================*/

#include "stm8s_type.h"


#define CAN_BASE_ADDRESS (0x5420)
#define CAN_PAGE_ADDRESS (CAN_BASE_ADDRESS+0x08)

#define CAN_MCR   (*(volatile char *)(CAN_BASE_ADDRESS+0x00))
#define CAN_MSR   (*(volatile char *)(CAN_BASE_ADDRESS+0x01))
#define CAN_TSR   (*(volatile char *)(CAN_BASE_ADDRESS+0x02))
#define CAN_TPR   (*(volatile char *)(CAN_BASE_ADDRESS+0x03))
#define CAN_RFR   (*(volatile char *)(CAN_BASE_ADDRESS+0x04))
#define CAN_IER   (*(volatile char *)(CAN_BASE_ADDRESS+0x05))
#define CAN_DGR   (*(volatile char *)(CAN_BASE_ADDRESS+0x06))
#define CAN_PSR   (*(volatile char *)(CAN_BASE_ADDRESS+0x07))

//pages 0,1,5: TxMailBox0, TxMailBox1, TxMailBox2
#define CAN_MCSR  (*(volatile char *)(CAN_PAGE_ADDRESS+0x00))
#define CAN_MDLCR (*(volatile char *)(CAN_PAGE_ADDRESS+0x01))
#define CAN_MIDR1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x02))
#define CAN_MIDR2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x03))
#define CAN_MIDR3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x04))
#define CAN_MIDR4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x05))
#define CAN_MDAR1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x06))
#define CAN_MDAR2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x07))
#define CAN_MDAR3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x08))
#define CAN_MDAR4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x09))
#define CAN_MDAR5 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0A))
#define CAN_MDAR6 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0B))
#define CAN_MDAR7 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0C))
#define CAN_MDAR8 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0D))
#define CAN_MTSRL (*(volatile char *)(CAN_PAGE_ADDRESS+0x0E))
#define CAN_MTSRH (*(volatile char *)(CAN_PAGE_ADDRESS+0x0F))

//page 6: Config/diagnostic
#define CAN_ESR   (*(volatile char *)(CAN_PAGE_ADDRESS+0x00))
#define CAN_EIER  (*(volatile char *)(CAN_PAGE_ADDRESS+0x01))
#define CAN_TECR  (*(volatile char *)(CAN_PAGE_ADDRESS+0x02))
#define CAN_RECR  (*(volatile char *)(CAN_PAGE_ADDRESS+0x03))
#define CAN_BTR1  (*(volatile char *)(CAN_PAGE_ADDRESS+0x04))
#define CAN_BTR2  (*(volatile char *)(CAN_PAGE_ADDRESS+0x05))
#define RESERVED0 (*(volatile char *)(CAN_PAGE_ADDRESS+0x06))
#define RESERVED1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x07))
#define CAN_FMR1  (*(volatile char *)(CAN_PAGE_ADDRESS+0x08))
#define CAN_FMR2  (*(volatile char *)(CAN_PAGE_ADDRESS+0x09))
#define CAN_FCR1  (*(volatile char *)(CAN_PAGE_ADDRESS+0x0A))
#define CAN_FCR2  (*(volatile char *)(CAN_PAGE_ADDRESS+0x0B))
#define CAN_FCR3  (*(volatile char *)(CAN_PAGE_ADDRESS+0x0C))
#define RESERVED2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0D))
#define RESERVED3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0E))
#define RESERVED4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0F))

//page7: Receive FIFO
#define CAN_MFMIR (*(volatile char *)(CAN_PAGE_ADDRESS+0x00))
#define CAN_MDLCR (*(volatile char *)(CAN_PAGE_ADDRESS+0x01))
#define CAN_MIDR1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x02))
#define CAN_MIDR2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x03))
#define CAN_MIDR3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x04))
#define CAN_MIDR4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x05))
#define CAN_MDAR1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x06))
#define CAN_MDAR2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x07))
#define CAN_MDAR3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x08))
#define CAN_MDAR4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x09))
#define CAN_MDAR5 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0A))
#define CAN_MDAR6 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0B))
#define CAN_MDAR7 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0C))
#define CAN_MDAR8 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0D))
#define CAN_MTSRL (*(volatile char *)(CAN_PAGE_ADDRESS+0x0E))
#define CAN_MTSRH (*(volatile char *)(CAN_PAGE_ADDRESS+0x0F))

//page2: Acceptance Filter 0, 1
#define CAN_F0R1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x00))
#define CAN_F0R2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x01))
#define CAN_F0R3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x02))
#define CAN_F0R4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x03))
#define CAN_F0R5 (*(volatile char *)(CAN_PAGE_ADDRESS+0x04))
#define CAN_F0R6 (*(volatile char *)(CAN_PAGE_ADDRESS+0x05))
#define CAN_F0R7 (*(volatile char *)(CAN_PAGE_ADDRESS+0x06))
#define CAN_F0R8 (*(volatile char *)(CAN_PAGE_ADDRESS+0x07))
#define CAN_F1R1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x08))
#define CAN_F1R2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x09))
#define CAN_F1R3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0A))
#define CAN_F1R4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0B))
#define CAN_F1R5 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0C))
#define CAN_F1R6 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0D))
#define CAN_F1R7 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0E))
#define CAN_F1R8 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0F))

//page3: Acceptance Filter 2, 3
#define CAN_F2R1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x00))
#define CAN_F2R2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x01))
#define CAN_F2R3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x02))
#define CAN_F2R4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x03))
#define CAN_F2R5 (*(volatile char *)(CAN_PAGE_ADDRESS+0x04))
#define CAN_F2R6 (*(volatile char *)(CAN_PAGE_ADDRESS+0x05))
#define CAN_F2R7 (*(volatile char *)(CAN_PAGE_ADDRESS+0x06))
#define CAN_F2R8 (*(volatile char *)(CAN_PAGE_ADDRESS+0x07))
#define CAN_F3R1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x08))
#define CAN_F3R2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x09))
#define CAN_F3R3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0A))
#define CAN_F3R4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0B))
#define CAN_F3R5 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0C))
#define CAN_F3R6 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0D))
#define CAN_F3R7 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0E))
#define CAN_F3R8 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0F))

//page4: Acceptance Filter 4, 5
#define CAN_F4R1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x00))
#define CAN_F4R2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x01))
#define CAN_F4R3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x02))
#define CAN_F4R4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x03))
#define CAN_F4R5 (*(volatile char *)(CAN_PAGE_ADDRESS+0x04))
#define CAN_F4R6 (*(volatile char *)(CAN_PAGE_ADDRESS+0x05))
#define CAN_F4R7 (*(volatile char *)(CAN_PAGE_ADDRESS+0x06))
#define CAN_F4R8 (*(volatile char *)(CAN_PAGE_ADDRESS+0x07))
#define CAN_F5R1 (*(volatile char *)(CAN_PAGE_ADDRESS+0x08))
#define CAN_F5R2 (*(volatile char *)(CAN_PAGE_ADDRESS+0x09))
#define CAN_F5R3 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0A))
#define CAN_F5R4 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0B))
#define CAN_F5R5 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0C))
#define CAN_F5R6 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0D))
#define CAN_F5R7 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0E))
#define CAN_F5R8 (*(volatile char *)(CAN_PAGE_ADDRESS+0x0F))

/*CAN Registers Reset Values*/
#define CANUART_MCR_RESET_VALUE   ((u8)0x02)
#define CANUART_IER_RESET_VALUE   ((u8)0x70)
#define CANUART_EIER_RESET_VALUE  ((u8)0x60)
#define CANUART_BTR1_RESET_VALUE  ((u8)0x40)
#define CANUART_BTR2_RESET_VALUE  ((u8)0x23)
#define CANUART_PSR_RESET_VALUE   ((u8)0x00)
#define CANUART_FMR1_RESET_VALUE  ((u8)0x00)
#define CANUART_FMR2_RESET_VALUE  ((u8)0xF0)
#define CANUART_FCR1_RESET_VALUE  ((u8)0x88)
#define CANUART_FCR2_RESET_VALUE  ((u8)0x00)
#define CANUART_FCR3_RESET_VALUE  ((u8)0x88)

/*CAN Registers Bits*/
#define CAN_TPR_TME0    ((u8)0x04)
#define CAN_TPR_TME1    ((u8)0x08)
#define CAN_TPR_TME2    ((u8)0x10)

#define CAN_TSR_RQCP0   ((u8)0x01)
#define CAN_TSR_RQCP1   ((u8)0x02)
#define CAN_TSR_RQCP2   ((u8)0x04)
#define CAN_TSR_TXOK0   ((u8)0x10)
#define CAN_TSR_TXOK1   ((u8)0x20)
#define CAN_TSR_TXOK2   ((u8)0x40)

#define CAN_MCSR_TXRQ   ((u8)0x01)
#define CAN_MCSR_ABRQ   ((u8)0x02)

#define CAN_RFR_FMP     ((u8)0x03)
#define CAN_RFR_RFOM    ((u8)0x20)

#define CAN_MIDR1_IDE   ((u8)0x40)
#define CAN_MIDR1_RTR   ((u8)0x20)
#define CAN_MIDR1_STDID ((u8)0x1F)
#define CAN_MIDR2_STDID ((u8)0xFC)

#define CAN_MDLCR_DLC   ((u8)0x0F)
#define CAN_IT_FMP0     ((u8)0x02) /* FIFOmessage pending */
#define CAN_IER_TMEIE   ((u8)0x01) /*Transmit Mailbox Empty interrupt*/
#define CAN_IER_FMPIE   ((u8)0x02)

#define CAN_DGR_LBKM    ((u8)0x01) /*Loopback Mode Enable*/
#define CAN_DGR_SILM    ((u8)0x02) /*Silent Mode Enable */

/* CAN Master Control Register bits */
#define CAN_MCR_INRQ     ((u8)0x01) /* Initialization request */
#define CAN_MCR_SLEEP    ((u8)0x02) /* Sleep mode request */
#define CAN_MCR_TXFP     ((u8)0x04) /* Transmit FIFO priority */
#define CAN_MCR_RFLM     ((u8)0x08) /* Receive FIFO locked mode */
#define CAN_MCR_NART     ((u8)0x10) /* No automatic retransmission */
#define CAN_MCR_AWUM     ((u8)0x20) /* Automatic wake up mode */
#define CAN_MCR_ABOM     ((u8)0x40) /* Automatic bus-off management */
#define CAN_MCR_TTCM     ((u8)0x80) /* time triggered communication */

/* CAN Master Status Register bits */
#define CAN_MSR_INAK     ((u8)0x01)    /* Initialization acknowledge */
#define CAN_MSR_SLAK     ((u8)0x02)    /* Sleep acknowledge */
#define CAN_MSR_WKUI     ((u8)0x08)    /* Wake-up interrupt */


/* CAN transmit constants */
#define CANTXFAILED      ((u8)0x00) /* CAN transmission failed */
#define CANTXOK          ((u8)0x01) /* CAN transmission succeeded */
#define CANTXPENDING     ((u8)0x02) /* CAN transmission pending */
#define CAN_NO_MB        ((u8)0x04) /* CAN cell did not provide an empty mailbox */
/* CAN sleep constants */
#define CANINITFAILED    ((u8)0x00) /* CAN initialization failed */
#define CANINITOK        ((u8)0x01) /* CAN initialization failed */

/* CAN filter scale */
// edits below.... ctarbert
#define CAN_FilterScale_8bit        ((u8)0x00) /* 8-bit filter scale */ //... correct for 8-bit
#define CAN_FilterScale_8_16bit     ((u8)0x01) /* 8 and 16-bit filter scale */ //... correct for 8- and 16-bit 
//#define CAN_FilterScale_16bit       ((u8)0x10) /* 16-bit filter scale */ //... wrong for 16-bit
#define CAN_FilterScale_16bit       ((u8)0x02) /* 16-bit filter scale */ //... correct for 16-bit equiv 0b0010
//#define CAN_FilterScale_32bit       ((u8)0x11) /* 32-bit filter scale */ //... wrong for 32-bit
#define CAN_FilterScale_32bit       ((u8)0x03) /* 32-bit filter scale */ //... correct for 32-bit equiv 0b0011
/* CAN operating mode */
#define CAN_Mode_Normal             ((u8)0x00)  /* normal mode */
#define CAN_Mode_LoopBack           ((u8)0x01)  /* loopback mode */
#define CAN_Mode_Silent             ((u8)0x02)  /* silent mode */
#define CAN_Mode_Silent_LoopBack    ((u8)0x03)  /* loopback combined with silent mode */

/* CAN synchronisation jump width */
#define CAN_SJW_1tq        ((u8)0x00)  /* 1 time quantum */
#define CAN_SJW_2tq        ((u8)0x01)  /* 2 time quantum */
#define CAN_SJW_3tq        ((u8)0x02)  /* 3 time quantum */
#define CAN_SJW_4tq        ((u8)0x03)  /* 4 time quantum */
/* time quantum in bit segment 1 */
#define CAN_BS1_1tq        ((u8)0x00)  /* 1 time quantum */
#define CAN_BS1_2tq        ((u8)0x01)  /* 2 time quantum */
#define CAN_BS1_3tq        ((u8)0x02)  /* 3 time quantum */
#define CAN_BS1_4tq        ((u8)0x03)  /* 4 time quantum */
#define CAN_BS1_5tq        ((u8)0x04)  /* 5 time quantum */
#define CAN_BS1_6tq        ((u8)0x05)  /* 6 time quantum */
#define CAN_BS1_7tq        ((u8)0x06)  /* 7 time quantum */
#define CAN_BS1_8tq        ((u8)0x07)  /* 8 time quantum */
#define CAN_BS1_9tq        ((u8)0x08)  /* 9 time quantum */
#define CAN_BS1_10tq       ((u8)0x09)  /* 10 time quantum */
#define CAN_BS1_11tq       ((u8)0x0A)  /* 11 time quantum */
#define CAN_BS1_12tq       ((u8)0x0B)  /* 12 time quantum */
#define CAN_BS1_13tq       ((u8)0x0C)  /* 13 time quantum */
#define CAN_BS1_14tq       ((u8)0x0D)  /* 14 time quantum */
#define CAN_BS1_15tq       ((u8)0x0E)  /* 15 time quantum */
#define CAN_BS1_16tq       ((u8)0x0F)  /* 16 time quantum */
/* time quantum in bit segment 2 */
#define CAN_BS2_1tq        ((u8)0x00)  /* 1 time quantum */
#define CAN_BS2_2tq        ((u8)0x01)  /* 2 time quantum */
#define CAN_BS2_3tq        ((u8)0x02)  /* 3 time quantum */
#define CAN_BS2_4tq        ((u8)0x03)  /* 4 time quantum */
#define CAN_BS2_5tq        ((u8)0x04)  /* 5 time quantum */
#define CAN_BS2_6tq        ((u8)0x05)  /* 6 time quantum */
#define CAN_BS2_7tq        ((u8)0x06)  /* 7 time quantum */
#define CAN_BS2_8tq        ((u8)0x07)  /* 8 time quantum */
/* CAN remote transmission request */
#define CAN_RTR_DATA       ((u8)0x00)  /* Data frame */
#define CAN_RTR_REMOTE     ((u8)0x01)  /* Remote frame */
/* CAN identifier type */
#define CAN_ID_STD        ((u8)0x00)  /* Standard Id */
#define CAN_ID_EXT        ((u8)0x01)  /* Extended Id */
/* CAN clock definition*/
#define CAN_CPU_CLK      ((u8)0x00)
#define CAN_EXT_CLK      ((u8)0x80)

typedef enum {FAILED = 0, PASSED = !FAILED} TestStatus;

/* CAN init structure definition */
typedef struct
{
  FunctionalState TTCM;
  FunctionalState ABOM;
  FunctionalState AWUM;
  FunctionalState NART;
  FunctionalState RFLM;
  FunctionalState TXFP;
  u8 Mode;
  u8 SJW;
  u8 BS1;
  u8 BS2;
  u8 Prescaler;
} CAN_InitTypeDef;

/* CAN filter init structure definition */
typedef struct
{
  u8 CAN_FilterNumber;
  u8 CAN_FilterMode;
  u8 CAN_FilterScale;
  u8 CAN_FilterIdHigh1;
  u8 CAN_FilterIdHigh2;
  u8 CAN_FilterIdLow1;
  u8 CAN_FilterIdLow2;
  u8 CAN_FilterMaskIdHigh1;
  u8 CAN_FilterMaskIdHigh2;
  u8 CAN_FilterMaskIdLow1;
  u8 CAN_FilterMaskIdLow2;
  FunctionalState CAN_FilterActivation;
} CAN_FilterInitTypeDef;

/* CAN Tx message structure definition */
typedef struct
{
  u16 StdId;
  u8 ExtId_28_24;
  u8 ExtId_23_16;
  u8 ExtId_15_8;
  u8 ExtId_7_0;
  u8 IDE;
  u8 RTR;
  u8 DLC;
  u8 Data[8];
} CanTxMsg;

/* CAN Rx message structure definition */
typedef struct
{
  u16 StdId;
  u8 ExtId_28_24;
  u8 ExtId_23_16;
  u8 ExtId_15_8;
  u8 ExtId_7_0;
  u8 IDE;
  u8 RTR;
  u8 DLC;
  u8 Data[8];
  u8 FMI;
} CanRxMsg;

/* Function prototypes */
void CAN_DeInit(void);
void CAN_testmode (u8 testmode);
u8 CAN_Init(CAN_InitTypeDef* CAN_InitStruct);
void CAN_FilterInit(CAN_FilterInitTypeDef* CAN_FilterInitStruct);
void CAN_StructInit(CAN_InitTypeDef* CAN_InitStruct);
u8 CAN_Transmit(CanTxMsg* TxMessage);
u8 CAN_TransmitStatus(u8 TransmitMailbox);
void CAN_CancelTransmit(void);
u8 CAN_MessagePending(void);
void CAN_FIFORelease(void);
void CAN_Receive(CanRxMsg* RxMessage);