/*===========================================================================
 * DESCRIPTION:
 *
 *  Prototype of firmware for STM8S microcontroller on GlueX FCAL PMT base. 
 *
 *  On receipt of a CAN message the microcontroller will either:
 *      - read from a DS3401 ID chip and send a CAN message containing the ID.
 *      - read one of the microcontroller's ADCs and send a CAN message
 *        containing the ADC value.
 *      - send a digital value to an LTC2630 DAC chip.
 *
 *  T. Smith
 *  C. Tarbert
 *
 *==========================================================================*/
 #pragma SRC
#include <intrist7.h>   /* Defines the _nop_() intrinsic function */
#include "stm8s_conf.h"
#include "stm8s_lib.h"
#include "firmware_config.h"
#include "CAN_Message.h"
#include "config_commands.h"
#include "read_ID.h"
#include "main.h"

/*****************************************************************************
 * Main program entry point.
 */

u8 G_BOARDID[8]={0,0,0,0,0,0,0,0};		// Global variable that will house board ID

//Global variables storing averaging info
u16 vcath_vals[32];
u16 curr_vals[32];


u8 TestPulserFlag=0;		// Flag that determines whether test pulser status

u8 HV_set1=0;           //Stores the HV value set to a base
u8 HV_set2=0;           //Stores the HV value set to a base

extern int jumpBL = 0;

at 0xFA00 void main_fw(void)   
{
	unsigned int i;
	TestPulserFlag = 0;
    jumpBL = 0;
	
    // initially configure all ports as output PP low slow
	// subsequent configuration should change ports to correct mode leaving
	// unused ports as output PP low slow
	// PC1 and PC2 should be left as inputs
    GPIO_Init(GPIOA, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port A
    GPIO_Init(GPIOB, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port B
    GPIO_Init(GPIOC, GPIO_PIN_ALL & (~(GPIO_PIN_1 | GPIO_PIN_2)), GPIO_MODE_OUT_PP_LOW_SLOW); // Port C
	GPIO_Init(GPIOD, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port D
    GPIO_Init(GPIOE, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port E
    GPIO_Init(GPIOF, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port F
    GPIO_Init(GPIOG, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port G
    GPIO_Init(GPIOH, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port H
    GPIO_Init(GPIOI, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port I
	
	// DAC chip select should be high
    GPIO_WriteHigh(DAC_PORT, DAC_CS_PIN); 
	
	// Try setting all unused ports (that don't have a pin out on 48-pin MCU to PP Low Slow
	// this makes no difference to power consumption
    GPIO_Init(GPIOI, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port I
	GPIO_Init(GPIOH, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port H
    GPIO_Init(GPIOC, GPIO_PIN_0, GPIO_MODE_OUT_PP_LOW_SLOW);   // Port C
		
    //Configure Clock
    Config_InternalClock();
	
	//1/13/11 DWB added CLK_HSITRIMR test to boost internal clock slightly
	//2nd time, copied firmware exactly and only made this change
	//Re-added on 3/30/11 for version 1.21, alternate version
	CLK->HSITRIMR = ((s8)  -1);

    // Configure CAN_TX, CAN_RX and CAN_STBY pins
    Config_IO_CAN();

	// Initialise G_BOARDID array to 0
	//Must read ID before configuring CAN - board ID needed to set up arbitration.
    for(i=0; i<8; i++){
		G_BOARDID[i] = 0x0;
    }
    // Fills G_BOARDID array with board ID 
    OWReadID();
	
	
    //Configure CAN - clocks, baudrate, filtering etc
    CAN_InitTransmit();
	
	
	// Configure LEDs
	//ConfigLEDs();

	// Configure ports for ADCs
    GPIO_Init(MON_PORT, MON_TEMP_PIN, GPIO_MODE_IN_FL_NO_IT);      // temp monitor
	GPIO_Init(MON_PORT, MON_CURR_PIN, GPIO_MODE_IN_FL_NO_IT);      // current monitor
    GPIO_Init(MON_PORT, MON_VDAC_PIN, GPIO_MODE_IN_FL_NO_IT);      // HV monitor DAC
    GPIO_Init(MON_PORT, MON_VCATH_PIN, GPIO_MODE_IN_FL_NO_IT);     // HV monitor photocathode
    GPIO_Init(MON_PORT, MON_VDY01_PIN, GPIO_MODE_IN_FL_NO_IT);     // HV monitor 1st dynode
    GPIO_Init(MON_PORT, MON_MVT_PIN, GPIO_MODE_IN_FL_NO_IT);       // HV monitor medium voltage top
    GPIO_Init(MON_PORT, MON_MVB_PIN, GPIO_MODE_IN_FL_NO_IT);       // HV monitor medium voltage bottom
	
	// Configure ports for HV enable bits...
    GPIO_Init(ENB_PORT, ENB_MVT_PIN, GPIO_MODE_OUT_PP_LOW_SLOW);
    GPIO_Init(ENB_PORT, ENB_MVB_PIN, GPIO_MODE_OUT_PP_LOW_SLOW);
    GPIO_Init(ENB_PORT, JAM_HVB_PIN, GPIO_MODE_OUT_PP_LOW_SLOW);
    GPIO_Init(ENB_PORT, ENB_HVB_PIN, GPIO_MODE_OUT_PP_LOW_SLOW);
    GPIO_Init(ENB_PORT, JAM_HVT_PIN, GPIO_MODE_OUT_PP_LOW_SLOW);
    GPIO_Init(ENB_PORT, ENB_HVT_PIN, GPIO_MODE_OUT_PP_LOW_SLOW);
	// Disable all to start with...
    GPIO_WriteLow(ENB_PORT, ENB_MVT_PIN);
    GPIO_WriteLow(ENB_PORT, ENB_MVB_PIN);
    GPIO_WriteLow(ENB_PORT, JAM_HVB_PIN);
    GPIO_WriteLow(ENB_PORT, ENB_HVB_PIN);
    GPIO_WriteLow(ENB_PORT, JAM_HVT_PIN);
    GPIO_WriteLow(ENB_PORT, ENB_HVT_PIN);
	
    //Every peripheral is ready to start communication
    
	//initialize to zero
    for(i=0; i<32; ++i) {
    curr_vals[i]=0;
    vcath_vals[i]=0;
    }
    
	// Debugging... indication that reset has occured...
    SetLEDs(0x00);
	// enable interrupts 
    FlashLEDs(2);
    enableInterrupts();
    
    while(1)
    {      
    
    
		delay(50);
		while( TestPulserFlag == 1 ){
			// If flag is set, fire test pulser.
			// Stops on receipt of a CAN message.
			
			GPIO_WriteHigh( TP_PORT, FIRE_TP_PIN );
			GPIO_WriteLow( TP_PORT, FIRE_TP_PIN );
			delay(5);
    
    
		}
    }
	
}
