/*===========================================================================
 * DESCRIPTION:
 *
 * This file implements CAN functions that mimick the STMicroelectronics'
 * STM8S library, but specifically for the CAN peripheral.
 * As CAN support is not yet available from the STM8S library, Raisonance
 * makes this file available for your convenience, although NO SUPPORT will
 * be provided for it.
 *
 *---------------------------------------------------------------------------
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *---------------------------------------------------------------------------
 * Copyright (c) Raisonance S.A.S., 2005-2008
 *==========================================================================*/

#include "stm8s_lib.h"
#include "stm8s_can.h"

/*****************************************************************CAN functions*************************************************/

void CAN_DeInit(void)
{
    /*page6*/
    CAN_PSR  = 0x06;
    CAN_EIER &= CANUART_EIER_RESET_VALUE;
    CAN_BTR1 = CANUART_BTR1_RESET_VALUE;
    CAN_BTR2 = CANUART_BTR2_RESET_VALUE;
    CAN_FMR1 = CANUART_FMR1_RESET_VALUE;
    CAN_FMR2 &= CANUART_FMR2_RESET_VALUE;
    CAN_FCR1 &= CANUART_FCR1_RESET_VALUE;
    CAN_FCR2 = CANUART_FCR2_RESET_VALUE;
    CAN_FCR3 &= CANUART_FCR3_RESET_VALUE;
    /*page0*/
    CAN_PSR  = 0x00;
    CAN_MCR = CANUART_MCR_RESET_VALUE;
    CAN_IER &= CANUART_IER_RESET_VALUE;
}

void CAN_StructInit(CAN_InitTypeDef* CAN_InitStruct)
{
    /* Reset CAN init structure parameters values */
    /* Initialize the time triggered communication mode */
    CAN_InitStruct->TTCM = DISABLE;
    /* Initialize the automatic bus-off management */
    CAN_InitStruct->ABOM = DISABLE;
    /* Initialize the automatic wake-up mode */
    CAN_InitStruct->AWUM = DISABLE;
    /* Initialize the no automatic retransmission */
    CAN_InitStruct->NART = DISABLE;
    /* Initialize the receive FIFO locked mode */
    CAN_InitStruct->RFLM = DISABLE;
    /* Initialize the transmit FIFO priority */
    CAN_InitStruct->TXFP = DISABLE;
    /* Initialize the CAN_Mode member */
    CAN_InitStruct->Mode = CAN_Mode_Normal;
    /* Initialize the CAN_SJW member */
    CAN_InitStruct->SJW = CAN_SJW_1tq;
    /* Initialize the CAN_BS1 member */
    CAN_InitStruct->BS1 = CAN_BS1_4tq;
    /* Initialize the CAN_BS2 member */
    CAN_InitStruct->BS2 = CAN_BS2_3tq;
    /* Initialize the CAN_Prescaler member */
    CAN_InitStruct->Prescaler = 1;
}

void CAN_testmode (u8 testmode)
{
    CAN_DGR &= ~(CAN_DGR_SILM | CAN_DGR_LBKM);     // set testmode
    CAN_DGR |=  (testmode & (CAN_DGR_SILM | CAN_DGR_LBKM));
}

u8 CAN_Init(CAN_InitTypeDef* CAN_InitStruct)
{
    u8 InitStatus = 0;
    // Request initialisation 
    CAN_MCR = CAN_MCR_INRQ;
    // Wait for the acknowledge 
//**********************!!!!!!
	    while ((CAN_MSR & CAN_MCR_INRQ)==0);
    // ...and check acknowledged/
		
	
    if ((CAN_MSR & CAN_MSR_INAK) == 0)
    {
        InitStatus = CANINITFAILED;
    }
	
    else
    {
        // Set the time triggered communication mode 
        if (CAN_InitStruct->TTCM == ENABLE)
        {
        CAN_MCR |= CAN_MCR_TTCM;
        }
        else
        {
        CAN_MCR &= (u8)(~CAN_MCR_TTCM);
        }
        // Set the automatic bus-off management 
        if (CAN_InitStruct->ABOM == ENABLE)
        {
        CAN_MCR |= CAN_MCR_ABOM;
        }
        else
        {
        CAN_MCR &= (u8)(~CAN_MCR_ABOM);
        }
        // Set the automatic wake-up mode 
        if (CAN_InitStruct->AWUM == ENABLE)
        {
        CAN_MCR |= CAN_MCR_AWUM;
        }
        else
        {
        CAN_MCR &= (u8)(~CAN_MCR_AWUM);
        }
        // Set the no automatic retransmission 
        if (CAN_InitStruct->NART == ENABLE)
        {
        CAN_MCR |= CAN_MCR_NART;
        }
        else
        {
        CAN_MCR &= (u8)(~CAN_MCR_NART);
        }
        // Set the receive FIFO locked mode 
        if (CAN_InitStruct->RFLM == ENABLE)
        {
        CAN_MCR |= CAN_MCR_RFLM;
        }
        else
        {
        CAN_MCR &= (u8)(~CAN_MCR_RFLM);
        }
        // Set the transmit FIFO priority 
        if (CAN_InitStruct->TXFP == ENABLE)
        {
        CAN_MCR |= CAN_MCR_TXFP;
        }
        else
        {
        CAN_MCR &= (u8)(~CAN_MCR_TXFP);
        }
        //Set the bit timing register 
        CAN_DGR |= (u8)(CAN_InitStruct->Mode);
        //page select
        CAN_PSR = 0x06;
        CAN_BTR1 = ((u8)(CAN_InitStruct->SJW << 6)) | ((u8)(CAN_InitStruct->Prescaler - 1));
        CAN_BTR2 = ((u8)CAN_InitStruct->BS1) | ((u8)(CAN_InitStruct->BS2 << 4));
        //back to page 0
        CAN_PSR = 0x00;

        InitStatus = CANINITOK;

        //IT: FIFO 0 msg pending, Transmit mbx empty
		//DWB REMOVED TMEIE, caused error, works without
        CAN_IER = (CAN_IER_FMPIE );// |CAN_IER_TMEIE);

    }
	
    /* At this step, return the status of initialization */
    return InitStatus;
}

void CAN_FilterInit(CAN_FilterInitTypeDef* CAN_FilterInitStruct)
{
	// Edited this function to correct typos.  Had to make sure
	// page select register was being set correctly (CAN_PSR).
	// c.tarbert 
	
    /*page select*/
    CAN_PSR = 0x06;
    /* Filter Deactivation */
    switch((u8)(CAN_FilterInitStruct->CAN_FilterNumber))
    {
    case 0:
        CAN_FCR1 &= ~(u8)(0x01);
        break;
    case 1:
        CAN_FCR1 &= ~(u8)(0x10);
        break;
    case 2:
        CAN_FCR2 &= ~(u8)(0x01);
        break;
    case 3:
        CAN_FCR2 &= ~(u8)(0x10);
        break;
    case 4:
        CAN_FCR3 &= ~(u8)(0x01);
        break;
    case 5:
        CAN_FCR3 &= ~(u8)(0x10);
        break;
    }
  /* Filter Scale */
    switch((u8)(CAN_FilterInitStruct->CAN_FilterNumber))
    {
    case 0:
        CAN_FCR1 |= (u8)((CAN_FilterInitStruct->CAN_FilterScale)<<1);
        break;
    case 1:
        CAN_FCR1 |= (u8)((CAN_FilterInitStruct->CAN_FilterScale)<<5);
        break;
    case 2:
        CAN_FCR2 |= (u8)((CAN_FilterInitStruct->CAN_FilterScale)<<1);
        break;
    case 3:
        CAN_FCR2 |= (u8)((CAN_FilterInitStruct->CAN_FilterScale)<<5);
        break;
    case 4:
        CAN_FCR3 |= (u8)((CAN_FilterInitStruct->CAN_FilterScale)<<1);
        break;
    case 5:
        CAN_FCR3 |= (u8)((CAN_FilterInitStruct->CAN_FilterScale)<<5);
        break;
    }

  if (CAN_FilterInitStruct->CAN_FilterScale == CAN_FilterScale_8bit)
  {
    switch((u8)(CAN_FilterInitStruct->CAN_FilterNumber))
    {
    case 0:
        /*page2*/
        CAN_PSR = 0x02;
        /* First 16-bit identifier and First 16-bit mask Or First 16-bit identifier and Second 16-bit identifier */
        CAN_F0R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F0R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F0R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F0R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        /* Second 16-bit identifier and Second 16-bit mask Or Third 16-bit identifier and Fourth 16-bit identifier */
        CAN_F0R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F0R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F0R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F0R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 1:
        /*page2*/
        CAN_PSR = 0x02;
         /* First 16-bit identifier and First 16-bit mask Or First 16-bit identifier and Second 16-bit identifier */
        CAN_F1R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F1R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F1R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F1R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        /* Second 16-bit identifier and Second 16-bit mask Or Third 16-bit identifier and Fourth 16-bit identifier */
        CAN_F1R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F1R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F1R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F1R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 2:
        /*page3*/
        CAN_PSR = 0x03;
         /* First 16-bit identifier and First 16-bit mask Or First 16-bit identifier and Second 16-bit identifier */
        CAN_F2R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F2R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F2R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F2R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        /* Second 16-bit identifier and Second 16-bit mask Or Third 16-bit identifier and Fourth 16-bit identifier */
        CAN_F2R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F2R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F2R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F2R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 3:
        /*page3*/
        CAN_PSR = 0x03;
         /* First 16-bit identifier and First 16-bit mask Or First 16-bit identifier and Second 16-bit identifier */
        CAN_F3R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F3R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F3R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F3R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        /* Second 16-bit identifier and Second 16-bit mask Or Third 16-bit identifier and Fourth 16-bit identifier */
        CAN_F3R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F3R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F3R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F3R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 4:
        /*page4*/
        CAN_PSR = 0x04;
         /* First 16-bit identifier and First 16-bit mask Or First 16-bit identifier and Second 16-bit identifier */
        CAN_F4R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F4R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F4R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F4R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        /* Second 16-bit identifier and Second 16-bit mask Or Third 16-bit identifier and Fourth 16-bit identifier */
        CAN_F4R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F4R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F4R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F4R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 5:
        /*page4*/
        CAN_PSR = 0x04;
         /* First 16-bit identifier and First 16-bit mask Or First 16-bit identifier and Second 16-bit identifier */
        CAN_F5R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F5R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F5R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F5R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        /* Second 16-bit identifier and Second 16-bit mask  Or Third 16-bit identifier and Fourth 16-bit identifier */
        CAN_F5R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F5R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F5R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F5R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    }
  }
  if (CAN_FilterInitStruct->CAN_FilterScale == CAN_FilterScale_8_16bit)
  {
    switch((u8)(CAN_FilterInitStruct->CAN_FilterNumber))
    {
    case 0:
        /*page2*/
        CAN_PSR = 0x02;
        CAN_F0R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F0R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F0R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F0R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F0R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F0R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F0R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F0R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 1:
        /*page2*/
        CAN_PSR = 0x02;
        CAN_F1R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F1R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F1R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F1R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F1R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F1R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F1R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F1R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 2:
        /*page3*/
        CAN_PSR = 0x03;
        CAN_F2R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F2R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F2R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F2R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F2R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F2R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F2R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F2R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 3:
        /*page3*/
        CAN_PSR = 0x03;
        CAN_F3R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F3R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F3R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F3R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F3R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F3R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F3R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F3R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 4:
        /*page4*/
        CAN_PSR = 0x04;
        CAN_F4R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F4R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F4R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F4R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F4R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F4R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F4R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F4R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 5:
        /*page4*/
        CAN_PSR = 0x04;
        CAN_F5R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F5R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F5R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F5R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F5R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F5R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F5R7 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F5R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    }
  }
  if (CAN_FilterInitStruct->CAN_FilterScale == CAN_FilterScale_16bit)
   {
     switch((u8)(CAN_FilterInitStruct->CAN_FilterNumber))
     {
     case 0:
        /*page2*/
        CAN_PSR = 0x02;
        CAN_F0R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F0R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F0R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F0R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F0R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F0R6 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F0R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F0R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 1:
        /*page2*/
        CAN_PSR = 0x02;
        CAN_F1R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F1R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F1R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F1R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F1R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F1R6 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F1R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F1R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 2:
        /*page3*/
        CAN_PSR = 0x03;
        CAN_F2R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F2R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F2R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F2R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F2R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F2R6 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F2R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F2R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 3:
        /*page3*/
        CAN_PSR = 0x03;
        CAN_F3R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F3R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F3R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F3R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F3R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F3R6 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F3R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F3R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 4:
        /*page4*/
        CAN_PSR = 0x04;
        CAN_F4R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F4R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F4R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F4R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F4R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F4R6 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F4R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F4R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 5:
        /*page4*/
        CAN_PSR = 0x04;
        CAN_F5R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F5R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F5R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F5R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F5R5 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F5R6 = (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F5R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F5R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    }
  }
  if (CAN_FilterInitStruct->CAN_FilterScale == CAN_FilterScale_32bit)
  {
    switch((u8)(CAN_FilterInitStruct->CAN_FilterNumber))
    {
    case 0:
        /*page2*/
        CAN_PSR = 0x02;
        CAN_F0R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F0R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F0R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F0R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F0R5 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F0R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F0R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F0R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 1:
        /*page2*/
        CAN_PSR = 0x02;
        CAN_F1R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F1R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F1R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F1R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F1R5 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F1R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F1R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F1R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 2:
        /*page3*/
        CAN_PSR = 0x03;
        CAN_F2R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F2R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F2R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F2R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F2R5 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F2R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F2R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F2R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 3:
        /*page3*/
        CAN_PSR = 0x03;
        CAN_F3R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F3R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F3R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F3R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F3R5 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F3R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F3R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F2R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 4:
        /*page4*/
        CAN_PSR = 0x04;
        CAN_F4R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F4R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F4R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F4R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F4R5 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F4R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F4R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F4R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    case 5:
        /*page4*/
        CAN_PSR = 0x04;
        CAN_F5R1 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow1);
        CAN_F5R2 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdLow2);
        CAN_F5R3 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh1);
        CAN_F5R4 |= (u8)(CAN_FilterInitStruct->CAN_FilterIdHigh2);
        CAN_F5R5 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow1);
        CAN_F5R6 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdLow2);
        CAN_F5R7 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh1);
        CAN_F5R8 = (u8)(CAN_FilterInitStruct->CAN_FilterMaskIdHigh2);
        break;
    }
  }
  /* Filter Mode */
    /*page6*/
    CAN_PSR = 0x06;
    switch((u8)(CAN_FilterInitStruct->CAN_FilterNumber))
    {
    case 0:
        CAN_FMR1 |= (u8)(CAN_FilterInitStruct->CAN_FilterMode);
        break;
    case 1:
        CAN_FMR1 |= (u8)((CAN_FilterInitStruct->CAN_FilterMode)<<2);
        break;
    case 2:
        CAN_FMR1 |= (u8)((CAN_FilterInitStruct->CAN_FilterMode)<<4);
        break;
    case 3:
        CAN_FMR1 |= (u8)((CAN_FilterInitStruct->CAN_FilterMode)<<6);
        break;
    case 4:
        CAN_FMR2 |= (u8)(CAN_FilterInitStruct->CAN_FilterMode);
        break;
    case 5:
        CAN_FMR2 |= (u8)((CAN_FilterInitStruct->CAN_FilterMode)<<2);
        break;
    }
  /* Filter activation */
  if (CAN_FilterInitStruct->CAN_FilterActivation == ENABLE)
  {
      switch((u8)(CAN_FilterInitStruct->CAN_FilterNumber))
    {
    case 0:
        CAN_FCR1 |= (u8)(0x01);
        break;
    case 1:
        CAN_FCR1 |= (u8)(0x10);
        break;
    case 2:
        CAN_FCR2 |= (u8)(0x01);
        break;
    case 3:
        CAN_FCR2 |= (u8)(0x10);
        break;
    case 4:
        CAN_FCR3 |= (u8)(0x01);
        break;
    case 5:
        CAN_FCR3 |= (u8)(0x10);
        break;
    }
    /*page0*/
    CAN_PSR = 0x00;
  }
}

u8 CAN_Transmit(CanTxMsg* TxMessage)
{
    u8 TransmitMailbox = 0;

    /* Select one empty transmit mailbox */
    if ((CAN_TPR&CAN_TPR_TME0) == CAN_TPR_TME0)
    {
        TransmitMailbox = 0;
    }
    else if ((CAN_TPR&CAN_TPR_TME1) == CAN_TPR_TME1)
    {
        TransmitMailbox = 1;
        /*page1*/
        CAN_PSR = 0x01;
    }
    else if ((CAN_TPR&CAN_TPR_TME2) == CAN_TPR_TME2)
    {
        TransmitMailbox = 2;
        /*page5*/
        CAN_PSR = 0x05;
    }
    else
    {
        TransmitMailbox = CAN_NO_MB;
    }
    if (TransmitMailbox != CAN_NO_MB)
    {
        /* Set up the Id */
        if (TxMessage->IDE == CAN_ID_STD)
        {
            TxMessage->StdId &= (u16)0x07FF;
            CAN_MIDR1 = (u8)(  (u8)( (TxMessage->StdId & (u16)(0x07C0) )>>6 )  |  (u8)( (TxMessage->RTR) <<5 )  |  (u8)( (TxMessage->IDE) <<6 )  );
            CAN_MIDR2 |= (u8)(  ( TxMessage->StdId & (u16)(0x003F) )<<2  );
        }
        else
        {
            TxMessage->ExtId_28_24 &= (u8)(0x1F);
            CAN_MIDR4 = (u8)(TxMessage->ExtId_7_0);
            CAN_MIDR3 = (u8)(TxMessage->ExtId_15_8);
            CAN_MIDR2 = (u8)(TxMessage->ExtId_23_16);
            CAN_MIDR1 = (u8)(TxMessage->ExtId_28_24) | (u8)(TxMessage->RTR <<5) | (u8)(TxMessage->IDE <<6);
        }

        /* Set up the data field */
        CAN_MDAR1 = TxMessage->Data[0];
        CAN_MDAR2 = TxMessage->Data[1];
        CAN_MDAR3 = TxMessage->Data[2];
        CAN_MDAR4 = TxMessage->Data[3];
        CAN_MDAR5 = TxMessage->Data[4];
        CAN_MDAR6 = TxMessage->Data[5];
        CAN_MDAR7 = TxMessage->Data[6];
        CAN_MDAR8 = TxMessage->Data[7];

        /* Set up the DLC */
        TxMessage->DLC &= (u8)0x0F;
        CAN_MDLCR &= (u8)(0xF8);
        CAN_MDLCR |= TxMessage->DLC;

        //CAN_IER |= CAN_IER_TMEIE;//Enable TME Interrupt /// disabling this c.tarbert
		CAN_IER &= 0xFE; // make sure bit 0 is set to 0 i.e. TMIE interrupt is disabled. c.tarbert
        /* Request transmission */
        CAN_MCSR |= CAN_MCSR_TXRQ;
        /*page0*/
        CAN_PSR = 0x00;
    }
    return TransmitMailbox;
}

u8 CAN_TransmitStatus(u8 TransmitMailbox)
{
    /* RQCP, TXOK and TME bits */
    u8 State = 0;

    switch (TransmitMailbox)
    {
        case (0): State |= (u8)((CAN_TSR & CAN_TSR_RQCP0) << 2);
        State |= (u8)((CAN_TSR & CAN_TSR_TXOK0) >> 3);
        State |= (u8)((CAN_TPR & CAN_TPR_TME0) >> 2);
        break;
        case (1): State |= (u8)((CAN_TSR & CAN_TSR_RQCP1) << 1);
        State |= (u8)((CAN_TSR & CAN_TSR_TXOK1) >> 4);
        State |= (u8)((CAN_TPR & CAN_TPR_TME1) >> 3);
        break;
        case (2): State |= (u8)(CAN_TSR & CAN_TSR_RQCP2);
        State |= (u8)((CAN_TSR & CAN_TSR_TXOK2) >> 5);
        State |= (u8)((CAN_TPR & CAN_TPR_TME2) >> 4);
        break;
        default:
        State = CANTXFAILED;
        break;
    }
    switch (State)
    {
        /* transmit pending  */
        case (0x0): State = CANTXPENDING;
        break;
        /* transmit failed  */
        case (0x5): State = CANTXFAILED;
        break;
        /* transmit succedeed  */
        case (0x7): State = CANTXOK;
        break;
        default:
        State = CANTXFAILED;
        break;
    }
    return State;
}

void CAN_CancelTransmit(void)
{
    /* abort transmission */
    CAN_MCSR |= CAN_MCSR_ABRQ;
}

u8 CAN_MessagePending(void)
{
    u8 MessagePending=0;

    MessagePending = (u8)(CAN_RFR&CAN_RFR_FMP);

    return MessagePending;
}

void CAN_FIFORelease(void)
{
    /* Release FIFO */
    CAN_RFR = CAN_RFR_RFOM;
}

void CAN_Receive(CanRxMsg* RxMessage)
{
    /*page7*/
    CAN_PSR = 0x07;
    /* Get the Id */
    RxMessage->IDE = (u8)(CAN_MIDR1&CAN_MIDR1_IDE);
    if (RxMessage->IDE == CAN_ID_STD)
    {
        RxMessage->StdId  = (u16)(  ( (u16)(CAN_MIDR2&CAN_MIDR2_STDID) )>>2  );
        RxMessage->StdId |= (u16)(  ( (u16)(CAN_MIDR1&CAN_MIDR1_STDID) )<<6  );
    }
    else
    {
        RxMessage->ExtId_7_0  = (u8)(CAN_MIDR4);
        RxMessage->ExtId_15_8 = (u8)(CAN_MIDR3);
        RxMessage->ExtId_23_16 = (u8)(CAN_MIDR2);
        RxMessage->ExtId_28_24 = (u8)(CAN_MIDR1&CAN_MIDR1_STDID);
    }

    RxMessage->RTR = (u8)(CAN_MIDR1&CAN_MIDR1_RTR);

    /* Get the DLC */
    RxMessage->DLC = (u8)(CAN_MDLCR&CAN_MDLCR_DLC);

    /* Get the FMI */
    RxMessage->FMI = (u8)(CAN_MFMIR);

    /* Get the data field */
    RxMessage->Data[0] = (u8)(CAN_MDAR1);
    RxMessage->Data[1] = (u8)(CAN_MDAR2);
    RxMessage->Data[2] = (u8)(CAN_MDAR3);
    RxMessage->Data[3] = (u8)(CAN_MDAR4);
    RxMessage->Data[4] = (u8)(CAN_MDAR5);
    RxMessage->Data[5] = (u8)(CAN_MDAR6);
    RxMessage->Data[6] = (u8)(CAN_MDAR7);
    RxMessage->Data[7] = (u8)(CAN_MDAR8);
    /*page0*/
    CAN_PSR = 0x00;
    /* Release the FIFO */
    CAN_FIFORelease();
}
